/* Compile with: gcc -o RunImage scope.c `sdl-config --cflags` `sdl-config --libs` */
#include <stdio.h>
#include <sys/stat.h>
#include "PLOTlib.c"
#include <string.h>
#include <math.h>

#define SWAP(a,b) tempr=(a) ; (a)=(b) ; (b)=tempr
/* ROX scope program */
struct stat alsafileinfo;

long i,ok;

char dummy;

SDL_Event event;

int reason,quit,mouse_x,mouse_y,mouse_button;

int mouse_x,mouse_y;
int icount,keypress,harm;
int dmode;
char mstring[100];

short* sampointer;


long pk_plus,pk_minus;
long peak_plus,peak_minus;
long nn,n,npoints,nhalf,ip,ipk;
long pairs_per_sec;
double pi,pi2,ppeak,fpeak,pmean_left,pmean_right;

double fcounted,thd_left,thd_right,ppeak_left,ppeak_right;

double vin_scale,tscale,fscale;
double dc_left,dc_right,rms_left,rms_right;
double vpeak_plus,vpeak_minus;
unsigned char datablock[33000];
double left[8192], right[8192], weight[8192];
double pout[8193];
double x[16385];

FILE* settingsfile;
FILE* wavefile;

char appdirname[512];
char settingsfilename[512];
char commandstring[512];
char recordstring[512];
char recordfilename[512];

char alsafilename[512];
char astatus[512];

long read_settingsfile(void);
long load_recording(void);
int poll_sld(void);
void draw_scope(void);
void draw_fft(void);
void generate_apodise_weights(void);
void transform(void);
void load_fft_array(void);
void fft(void);
void bit_rev(void);
void apodise(void);
double dB0(double);
double find_counter_frequency(void);
void analyse_spectrum(void);

void alsa_status(void);

int main(int argc,char*argv[])
{
  dmode=0;
  
  sampointer=(short*)datablock;
  
  pi=3.1415927; pi2=2.0*pi;
  pairs_per_sec=48000;
  n=8192;  npoints=n;
  nn=2*n;  nhalf=n/2;
  generate_apodise_weights();
  /* printf("ROXscope v0.1\nJim Lesurf Dec 2012\n\n"); */
  i=argc;
  /* printf("Number of arguments given = %ld\n\n",i); */
  
  sprintf(appdirname,"%s\0",argv[0]);
  ok=(long)strrchr(appdirname,'/')-(long)appdirname;
  appdirname[ok]=(char)0;
  /* printf("Application directory = %s\n",appdirname); */
  sprintf(settingsfilename,"%s/%s\0",appdirname,"Settings\0");
  /* printf("Settings file at %s\n",settingsfilename); */
  
  ok=read_settingsfile();
  
  sprintf(recordfilename,"%s/audio.wav\0",appdirname);
  /* printf("recordings to \n = %s\n\n",recordfilename); */
  sprintf(commandstring,"%s %s\n",recordstring,recordfilename);
  /* printf("command \n = %s\n\n",commandstring); */
  
  alsa_status();
  
  generate_apodise_weights();
  
  PLOT_open_window(1100,520);
  
  tscale=1.0; fscale=1.0;
  
  icount=0;
   do
   {
     
     reason=poll_sld();
     system(commandstring);
     SDL_Delay(100);
     ok=load_recording();
     fcounted=find_counter_frequency();
     /* printf("fcounted = %6.3lf kHz\n",fcounted); */
     
     PLOT_setRGB(50,50,100);
     PLOT_rectangle_fill(5,5,1090,890);
     PLOT_setRGB(255,255,255);
     PLOT_rectangle(5,5,1090,890);
     
     draw_scope();
     draw_fft();
     
     PLOT_setRGB(200,200,200);
     PLOT_rectangle(80,482,800,20);
     PLOT_setRGB(150,255,255);
     PLOT_string(100,480,"ROX mini scope v1.1 Jan 2013");
     PLOT_string(400,480,"'s' or 'f' keys alter time displayed : 'm' toggles mode");
     PLOT_string(50,5,recordstring);
     PLOT_string(450,5,astatus);
     
     PLOT_show();
     
     if(reason==3)
     {
       sprintf(mstring,"Quit chosen");
       PLOT_string(100,50,mstring);
       PLOT_show();
       icount=10000;
     }
     
     if(reason==1)
     {
       sprintf(mstring,"key %i [%c] pressed",keypress,(char)keypress);
       PLOT_string(100,50,mstring);
       PLOT_show();
    
       if('f'==(char)keypress)
       {
         tscale=2.0*tscale;
         
         if(tscale>16.0)
         {
           tscale=16.0;
         }
         
       }
       
       if('s'==(char)keypress)
       {
         tscale=0.5*tscale;
         if (tscale<0.125)
         {
           tscale=0.125;
         }
       }
       
       if('m'==(char)keypress)
       {
         if(dmode==0)
         {
           dmode=1;
         }
         else
         {
           dmode=0;
         }
       }
     }
     
      SDL_Delay(100);
      icount++;
   } while (quit==0);
  
  SDL_Delay(1000);
  
  printf(" *Done!*\n");
  
}

void alsa_status(void)
{
  long afilesize,wal;
  sprintf(alsafilename,"%s/alsa_config",appdirname);
  stat(alsafilename,&alsafileinfo);
  afilesize=(long)alsafileinfo.st_size;
  
  if(afilesize>1)
  {
    
    sprintf(astatus,"alsactl -f %s restore",alsafilename);
    
    system(astatus);
    
    wal=0;
    do
    {
      log(2.0);
      wal++;
    } while (wal<10000000);
    printf("command %s\n",astatus);
  }
  else
  {
    sprintf(astatus,"ALSA default");
  }
  
}

void draw_fft(void)
{
  int x0_fft,y0_fft,h_fft,w_fft,itick;
  int xt0,yt0,xt,yt,yt_step;
  int ix,iy;
  
  double pa,pb,xfft_scale,yfft_scale,fstep,ftick;
  
  char ds_string[128];
  
  x0_fft=80; y0_fft=100; h_fft=350; w_fft=800;
  
  xt0=x0_fft + w_fft + 10; yt0=y0_fft + h_fft - 270; yt_step=15;
  
  xfft_scale=((double)w_fft)/8192.0;
  yfft_scale=5.0;
  
  fstep=2.0/fscale; ftick=(double)w_fft/12.0;
  
  PLOT_setRGB(0,30,0);
  PLOT_rectangle_fill(xt0-5,yt0-100,204,120);
  PLOT_setRGB(100,150,100);
  PLOT_rectangle(xt0-5,yt0-100,204,120);
  
  if(dmode==1)
  {
  
  PLOT_setRGB(0,30,0);
  PLOT_rectangle_fill(x0_fft-70,y0_fft-20,w_fft+70,h_fft+40);
  PLOT_setRGB(100,150,100);
  PLOT_rectangle(x0_fft,y0_fft,w_fft,h_fft);
  
  itick=0;
  do
  {
    ix=x0_fft; iy=(int)(y0_fft+yfft_scale*(67-10*itick));
    PLOT_move(ix-10,iy);
    PLOT_draw(ix+w_fft,iy);
    itick++;
  } while(itick<7);
  
  itick=0;
  do
  {
    ix=x0_fft+(int)((double)itick*ftick);
    PLOT_move(ix,y0_fft-5);
    PLOT_draw(ix,y0_fft+h_fft);
    itick++;
  } while (itick<12);
  
  }
  
  
  PLOT_setRGB(255,255,255);
  
  if(dmode==1)
  {
  
  itick=0;
  do
  {
    ix=x0_fft; iy=(int)(y0_fft+yfft_scale*(67-10*itick));
    sprintf(ds_string,"%4d",-10*itick);
    PLOT_string(ix-50,iy-12,ds_string);
    itick++;
  } while(itick<7);
  
  
  ix=x0_fft; iy=(int)(y0_fft+150);
  
  PLOT_string(ix-50,iy-4,"dBV");
  
  }
  
  ix=x0_fft+w_fft-30; iy=y0_fft-25;
  if(dmode==1)
  {
  PLOT_string(ix,iy,"kHz");
  }
  yt0-=5;
  sprintf(ds_string," Freq count = %6.3lf kHz",fcounted);
  PLOT_string(xt0,yt0,ds_string);
  yt0-=yt_step;
  sprintf(ds_string," FFT peak   = %6.3lf kHz",fpeak*0.001);
  PLOT_string(xt0,yt0,ds_string);
  yt0-=2*yt_step;
  PLOT_string(xt0,yt0," Level of max component");
  yt0-=yt_step;
  PLOT_string(xt0,yt0,"          left    right");
  yt0-=yt_step;
  sprintf(ds_string," dBVpk %7.2lf  %7.2lf",dB0(ppeak_left)-3.2,dB0(ppeak_right)-3.2);
  PLOT_string(xt0,yt0,ds_string);
  
  yt0-=yt_step;
  sprintf(ds_string," THD[%%]  %6.3lf   %6.3lf",thd_left,thd_right);
  PLOT_string(xt0,yt0,ds_string);
  yt0-=yt_step;
  
  if(dmode==1)
  {
  itick=0;
  do
  {
    ix=x0_fft+(int)((double)itick*ftick);
    sprintf(ds_string,"%5.2lf",16.0*(double)(itick-6));
    PLOT_string(ix-20,y0_fft-25,ds_string);
    itick++;
  } while (itick<12);
  }
  
  apodise();
  transform();
  analyse_spectrum();
  pa=0.0; pb=1000000.0;
  
  ix=0;
  do
  {
    if(pout[ix]>pa) pa=pout[ix];
    
    if(pout[ix]<pb) pb=pout[ix];
    
    ix++;
  } while (ix<4000);
  
  /* printf("spec %4.2lf  %4.2lf\n",dB0(pa),dB0(pb)); */
  
  if(dmode==1)
  {
  
  PLOT_setRGB(255,100,100);
  itick=0;
  ix=x0_fft; iy=(int)(yfft_scale*(dB0(pout[itick])+67.0));
  if(iy<0) iy=0;
  if(iy>h_fft)iy=h_fft;
  
  PLOT_move(ix,y0_fft+iy);
  itick=1;
  do
  {
    ix=x0_fft+(int)(xfft_scale*(double)itick); iy=yfft_scale*(dB0(pout[itick])+67.0);
    if(iy<0) iy=0;
    if(iy>h_fft)iy=h_fft;
    PLOT_draw(ix,y0_fft+iy);
    itick++;
  } while (itick<8192);
  
  
  
  
  }
  
}

void draw_scope(void)
{
  int x0_scope,y0_scope,h_scope,w_scope,h_tick,itick;
  int xt0,yt0,xt,yt,yt_step;
  int ix,iy;
  
  char ds_string[128];
  
  double vscale,vrange,vstep;
  
  x0_scope=80; y0_scope=250; h_scope=200; w_scope=800;
  h_tick=h_scope/5;
  
  
  
  xt0=x0_scope + w_scope + 10; yt0=y0_scope + h_scope - 20; yt_step=15;
  
  vrange=2.5;
  
  if(( vpeak_plus<1.25 ) && (vpeak_minus>(-1.25)) )
  {
    vrange=1.25;
  }
  
  if(( vpeak_plus<0.45) && (vpeak_minus>(-0.45)) )
  {
    vrange=0.5;
  }
  
  if( (vpeak_plus<0.095) && (vpeak_minus>(-0.095)) )
  {
    vrange=0.1;
  }
  
  vstep=vrange/5.0;
  vscale=((double)h_scope)/vrange;
  
  PLOT_setRGB(0,30,0);
  PLOT_rectangle_fill(xt0-5,yt0-100,204,120);
  PLOT_setRGB(100,150,100);
  PLOT_rectangle(xt0-5,yt0-100,204,120);
  
  if(dmode==0)
  {
  
  PLOT_setRGB(0,30,0);
  PLOT_rectangle_fill(x0_scope-70,y0_scope-h_scope-20,w_scope+70,2*h_scope+40);
  
  
  PLOT_setRGB(100,150,100);
  PLOT_rectangle(x0_scope,y0_scope-h_scope,w_scope,2*h_scope);
  
  itick=-4;
  do
  {
    PLOT_move(x0_scope-10,y0_scope+itick*h_tick);
    PLOT_draw(x0_scope+w_scope,y0_scope+itick*h_tick);
    itick++;
  } while (itick<5);
  
  itick=0;
  do
  {
    ix=x0_scope+itick*192;
    PLOT_move(ix,y0_scope-h_scope-8);
    PLOT_draw(ix,y0_scope+h_scope);
    itick++;
  } while (itick<5);
  
  PLOT_setRGB(255,255,255);
  
  PLOT_string(x0_scope-70,y0_scope+8,"Volts");
  
  PLOT_string(x0_scope+w_scope/2-120,y0_scope-h_scope-24,"ms");
  
  itick=-5;
  do
  {
    sprintf(ds_string,"%7.3lf",vstep*(double)itick);
    PLOT_string(x0_scope-70,y0_scope+itick*h_tick-12,ds_string);
    itick++;
  } while (itick<6);
  
  
  
  i=0;
  do
  {
    sprintf(ds_string,"%5.2lf",4.0*((double)i)/tscale);
    
    PLOT_string(x0_scope+i*192-20,y0_scope-h_scope-24,ds_string);
    i++;
  } while (i<5);

  }
  PLOT_setRGB(255,255,255);
  yt0-=5;
  sprintf(ds_string,"Volts: left    right");
  PLOT_string(xt0,yt0,ds_string);
  yt0-=yt_step;
  sprintf(ds_string,"rms = %7.4lf %7.4lf",rms_left,rms_right);
  PLOT_string(xt0,yt0,ds_string);
  yt0-=yt_step;
  sprintf(ds_string," dc = %7.4lf %7.4lf",dc_left,dc_right);
  PLOT_string(xt0,yt0,ds_string);
  
  yt0-=3*yt_step;
  sprintf(ds_string," max  %7.3lf [ %6ld ]",vpeak_plus,peak_plus);
  PLOT_string(xt0,yt0,ds_string);
  yt0-=yt_step;
  sprintf(ds_string," min  %7.3lf [ %6ld ]",vpeak_minus,peak_minus);
  PLOT_string(xt0,yt0,ds_string);
  
  yt0-=2*yt_step;
  if(peak_plus>32000)
  {
    PLOT_setRGB(255,50,50);
    PLOT_string(xt0+10,yt0,"* * * CLIPPING! * * *");
  }
  
  if(peak_minus<-32000)
  {
    PLOT_setRGB(255,50,50);
    PLOT_string(xt0+10,yt0,"* * * CLIPPING! * * *");
  }
  
  if(dmode==0)
  {
  
  PLOT_setRGB(255,100,100);
  itick=0;
  PLOT_move(x0_scope,y0_scope+(int)(vscale*left[itick]));
  itick=1;
  ix=(int)(tscale*(double)itick);
  
  while(ix<w_scope)
  {
    PLOT_draw(x0_scope+ix,y0_scope+(int)(vscale*left[itick]));
    itick++;
    ix=(int)(tscale*(double)itick);
  }
  
  
  PLOT_setRGB(100,100,255);
  itick=0;
  PLOT_move(x0_scope,y0_scope+(int)(vscale*right[itick]));
  itick=1;
  ix=(int)(tscale*(double)itick);
  
  while(ix<w_scope)
  {
    PLOT_draw(x0_scope+ix,y0_scope+(int)(vscale*right[itick]));
    itick++;
    ix=(int)(tscale*(double)itick);
  }
  }
  
}

int poll_sld(void)
{
  int poll_result;
  poll_result=0;
  while(SDL_PollEvent(&event))
  {
    
      if( event.type == SDL_KEYDOWN )
      {
        keypress=event.key.keysym.sym;
        poll_result=1;
      }
    
      if( event.type == SDL_MOUSEBUTTONDOWN )
      {
        mouse_x=event.button.x;
        mouse_y=event.button.y;
        mouse_button=event.button.button;
        poll_result=2;
      }
      if ( event.type == SDL_QUIT )
      {
        quit=1;
        poll_result=3;
      }
  }
  return poll_result;
}

void analyse_spectrum(void)
{
  
  long dosp,id,idc,hno;
  harm=0;
  thd_left=0.0; thd_right=0.0;
  dosp=0;
  
  ppeak=0.0; fpeak=0.0;
  i=0;
  do
  {
    if(pout[i]>ppeak)
    {
      ppeak=pout[i];
      ipk=i;
    }
    
    i++;
  } while (i<4095);
  
  fpeak=(24000.0/4096.0)*(double)ipk;
  
  
  
  if(fpeak>15.0)
  {
    if(fpeak<10800.0)
    {
      dosp=1;
      harm=(long)(21005.0/fpeak);
      if (harm>10) harm=10;
    }
  }
  
  
  if( dosp==0)
  {
    ppeak_left=0.0; ppeak_right=0.0;
    id=ipk-2; 
    if(id<0) id=0;
    if (id>4090) id=4090;
    idc=0;
    do
    {
      ppeak_left+=pout[id];
      
      id++;
      idc++;
    } while (idc<5);
  }
  
  
  if (dosp==1)
  {
    ppeak_left=0.0; ppeak_right=0.0;
    
    id=ipk-2; idc=0;
    do
    {
      ppeak_left+=pout[id];
      
      id++;
      idc++;
    } while (idc<5);
    
    
    if(ppeak_left>0.00000001)
    {
      
      hno=2;
      do
      {
        id=ipk*hno-2; idc=0;
        do
        {
          thd_left+=pout[id];
          id++;
          idc++;
        }while (idc<5);
        hno++;
      } while (hno<=harm);
      
      if (thd_left>0.0)
      {
        thd_left=thd_left/ppeak_left;
        thd_left=100.0*sqrt(thd_left);
      }
      else
      {
        thd_left=0.0;
      }
    }
    
    
    
    
    
    
    
 
    
  }
    
    
  
}



double find_counter_frequency(void)
{
  /* This proceedure employs a fringe counting method to
     determine the frequency of the periodic signal.
     It then sets f0 to be the found value for whichever
     channel has the highest power level.
  */
  long last_left,last_right, first_left,first_right;
  long ii,detect_left,detect_right,count_left,count_right;
  double ffound,f0_left,f0_right;
  last_left=-10;  last_right=-10;
  count_left=0;   count_right=0;
  ii=1;
  do
  {
    detect_left=0;
    if (left[ii-1]<=0)
    {
      if (left[ii]>0)
      {
        detect_left=1;
        
        if(last_left<0)
        {
          first_left=ii;
          last_left=detect_left;
        }
        else
        {
          if ((ii-last_left>4))
          {
            last_left=ii;
            count_left++;
          }
        }
      }
    }
      
    detect_right=0;
    if (right[ii-1]<=0)
    {
      if (right[ii]>0)
      {
        detect_right=1;
        
        if(last_right<0)
        {
          first_right=ii;
          last_right=detect_right;
        }
        else
        {
          if ((ii-last_right>4))
          {
            last_right=ii;
            count_right++;
          }
        }
      }
    }
      ii++;
    } while(ii<8192);
    
    /*
    printf("Left  %ld to %ld has %ld cycles\n",first_left,last_left,count_left);
    printf("Right %ld to %ld has %ld cycles\n",first_right,last_right,count_right);
    
    */
    
    ffound=(double)(last_left-first_left);
    
    if(count_left>0)
    {
      ffound=ffound/(double)(count_left);
      f0_left=(double)pairs_per_sec/ffound;
    }
    else
    {
      f0_left=0.0;
    }
    
    ffound=(double)(last_right-first_right);
    
    if(count_right>0)
    {
      
      ffound=ffound/(double)(count_right);
      f0_right=(double)pairs_per_sec/ffound;
    }
    else
    {
      f0_right=0.0;
    }
    
    /*
    printf("Frequencies %6.2lf Hz Left  %6.2lf Right\n",f0_left,f0_right);  
    */  
    if(pmean_left>pmean_right)
    {
      ffound=f0_left;
      /* printf("Left chosen\n"); */
    }
    else
    {
      ffound=f0_right;
     /*  printf("Right chosen\n"); */
    }
  
  return ffound*0.001;
}



long load_recording(void)
{
  long size_read;
  long icut,icut2;
  long io,ii,lt,rt;
  short sleft,sright;
  double samscale,left_dc,right_dc;
  
  icut=32768; icut2=icut*2;
  samscale=vin_scale/(double)icut;
  dc_left=0.0; dc_right=0.0;
  rms_left=0.0; rms_right=0.0;
  peak_plus=0; peak_minus=0;
  wavefile=fopen(recordfilename,"rd");
  
  fseek(wavefile,115244,0);
  
  size_read=fread(datablock,sizeof(unsigned char),32900,wavefile);
  
  ii=0; io=0; left_dc=0.0; right_dc=0.0;
  
  do
  {
    lt=(long)sampointer[ii]; rt=(long)sampointer[ii+1];
    
    left[io]=samscale*(double)lt;
    right[io]=samscale*(double)rt;
    
    if(lt>peak_plus)
    {
      peak_plus=lt;
    }
    if(lt<peak_minus)
    {
      peak_minus=lt;
    }
    
    if(rt>peak_plus)
    {
      peak_plus=rt;
    }
    if(rt<peak_minus)
    {
      peak_minus=rt;
    }
    
    left_dc+=(double)left[io]; right_dc+=(double)right[io];
    ii+=2;
    io++;
    
  } while (io<8192);
  
  left_dc=left_dc/8192.0; right_dc=right_dc/8192.0;
  
  dc_left=left_dc; dc_right=right_dc;
  
  io=0;
  do
  {
    
    left[io]-=left_dc;  right[io]-=right_dc;
    
    rms_right+=right[io]*right[io];
    rms_left+=left[io]*left[io];
    
    io++;
  } while (io<8192);
  
  fclose(wavefile);
  
  
  rms_left=rms_left/8192.0;
  rms_right=rms_right/8192.0;
  
  rms_left=sqrt(rms_left);
  rms_right=sqrt(rms_right);
  
  vpeak_plus=samscale*(double)peak_plus;
  vpeak_minus=samscale*(double)peak_minus;
  /*
  printf(" sample rms %8.4lf  %8.4lf  dc  %8.4lf  %8.4lf\n",rms_left,rms_right,dc_left,dc_right);
  printf(" sample range %ld to %ld\n",peak_plus,peak_minus);
  
  printf("Voltages rms %8.4lf  %8.4lf V\n",rms_left*vin_scale,rms_right*vin_scale);
  printf(" range %8.4lf to %8.4lf V\n",vin_scale*((double)peak_plus)/32768.0, vin_scale*((double)peak_minus)/32768.0);
  */
  
}

void save_powerspectrum(double* onearray)
{
  long icount,idc,idd;
  double p1,p2;
  icount=1; idc=1; idd=16384;
  do
  {
    p1=x[idc];
    idc++;
    p2=x[idc];
    idc++;
    p1=p1*p1+p2*p2;
    
    if (p1>ppeak)
    {
      ppeak=p1;
      ipk=icount;
    }
    onearray[icount+nhalf-1]=p1;
    
    
    
    p1=x[idd];
    idd--;
    p2=x[idd];
    idd--;
    p1=p1*p1+p2*p2;
    
    if (p1>ppeak)
    {
      ppeak=p1;
      ipk=icount;
    }
    onearray[nhalf-icount]=p1;
    
    icount++;
    
  } while (icount<nhalf);
}

void load_fft_array(void)
{
  long icount,icd;
  icount=0; icd=1;
  x[0]=0.0;
  do
  {
    
    x[icd]=left[icount];
    icd++;
    x[icd]=-right[icount];
    icd++;
    icount++;
  } while (icount<npoints);
  
  
}

void transform(void)
{
  ppeak=0.0;  fpeak=0.0;
  load_fft_array();
  fft();
  save_powerspectrum(pout);
  
  
}

void generate_apodise_weights(void)
{
  long iwf,iwb;
  double weight_now,wscale,xplace;
  FILE* report;
  iwf=0; iwb=8191; wscale=0.0;
    do
    {
      xplace=(double)iwf;
      xplace=pi2*xplace/8191.0;
      weight_now= 0.35875-0.48829*cos(xplace)+0.14128*cos(2.0*xplace)-0.01168*cos(3.0*xplace);
      weight[iwf]=weight_now;
      weight[iwb]=weight_now;
      wscale+=weight_now;
      iwf++;
      iwb--;
    } while (iwf<iwb);
    wscale=wscale/((double)(iwf-1));
    iwf=0;
    do
    {
      weight[iwf]=weight[iwf]/wscale;
      iwf++;
    } while (iwf<8192);
}

void fft(void)
{
  long mmax,istep,m,i,j;
  long ibig,ismall,jbig,jsmall;
  double theta,wpi,wpr;
  double wi,wr,tr,ti,wtemp;
  
  ibig=-99;  jbig=-99;
  ismall=99; jsmall=99;

  mmax=2;
  theta=2.0*pi/mmax;
  
  bit_rev();
  
  while ( nn > mmax )
  {
   istep=2*mmax;
   theta=2.0*pi/mmax;
   wpr=-2.0*sin(0.5*theta)*sin(0.5*theta);
   wpi=sin(theta);
   wr=1.0;
   wi=0.0;
   m=1;
   do
   {
     i=m;
     do
     {
       j=i+mmax;
       
       if(i>ibig)
       {
         ibig=i;
       }
       if(i<ismall)
       {
         ismall=i;
       }
       if (j>jbig)
       {
         jbig=j;
       }
       if (j<jsmall)
       {
         jsmall=j;
       }
       
       
       tr=wr*x[j]-wi*x[j+1];
       ti=wr*x[j+1]+wi*x[j];
       x[j]=x[i]-tr;
       x[j+1]=x[i+1]-ti;
       x[i]=x[i]+tr;
       x[i+1]=x[i+1]+ti;
       i+=istep;
     } while ( i < nn );
     wtemp=wr;
     wr=wr*wpr-wi*wpi+wr;
     wi=wi*wpr+wtemp*wpi+wi;
     m+=2;
   } while ( m < mmax );
   mmax=istep;   
  }
  
}


void bit_rev(void)
{
  long ibr,jbr,mbr;
  double tr,ti;
  jbr=1;
  ibr=1;
  do
  {
    if ( jbr > ibr )
    {
      tr=x[jbr];
      ti=x[jbr+1];
      x[jbr]=x[ibr];
      x[jbr+1]=x[ibr+1];
      x[ibr]=tr;
      x[ibr+1]=ti;
    }
    mbr=nn/2;
    while ( mbr >= 2 && jbr > mbr )
    {
      jbr-=mbr;
      mbr=mbr/2;
    }
    jbr=jbr+mbr;   
    ibr+=2;
  } while ( ibr < nn );
}

double dB0(double pin)
{
  double answer;
  if (pin<0.000000001)
  {
    pin=0.000000001;
  }
  answer=10.0*log10(pin)-71.9;
  return answer;
}

void apodise(void)
{
  long iap;
  iap=0;
  do
  {
    left[iap]=weight[iap]*left[iap];
    right[iap]=weight[iap]*right[iap];
    iap++;
  } while (iap<8192);
}

long read_settingsfile(void)
{
  long result,ir,iq;
  char chr;
  sprintf(settingsfilename,"%s/settings\0",appdirname);
  printf("\nSettings file at %s\n",settingsfilename);

  settingsfile=fopen(settingsfilename,"rt");
  
  ir=0; iq=0;
  do
  {
    fscanf(settingsfile,"%c",&chr);
    
    if(16<(unsigned int)chr)
    {
      recordstring[ir]=chr;
      ir++;
    }
    else
    {
      recordstring[ir]=(char)0;
      iq=1;
      
    }
  } while (iq==0);
    
  printf("read  *%s*\n",recordstring);
  
  fscanf(settingsfile,"%lf",&vin_scale);
  
  printf("Voltage scale factor = %7.3lf volts for full scale\n",vin_scale);
  
  fclose(settingsfile);
  result=1;
  return result;
}

